function [T,K] = doChordID(testFileList, scratchDir, resultsDir, prefix, ...
                        suffix, skip)
% fl = doChordID(testFileList, scratchDir, resultsDir)
%    Perform testing for MIREX08 chord recognition.
%      testFileList is a list of test files
%      scratchDir is the temporary directory where trained models live
%      resultsDir is where to write the per-file results
% 2008-08-21 Dan Ellis dpwe@ee.columbia.edu

% doChordID "/path/to/testFileList.txt" "/path/to/scratchDir" "/path/to/resultsDir"

if nargin < 4; prefix = ''; end
if nargin < 5; suffix = ''; end
if nargin < 6; skip = 0; end

% Do we use key normalization?
keynorm = 1;
%keycheat = 'KEYS';
keycheat = '';

if keynorm == 0
  disp('Key normalization DISABLED');
end

% make sure path is OK
if exist('viterbi_path') ~= 2
  addpath('KPMtools');
  addpath('KPMstats');
  addpath('KPMhmm');
end

% Load models
%modelfile = fullfile(scratchDir,'MODELS');
modelfile = 'MODELS-180tracks';   % load from CWD
load(modelfile); % loads M, txnn

tfl = listfileread(testFileList);

% Calculate features for test item
tstf = calcbeatchroma(testFileList,prefix,suffix,scratchDir,...
                      {'-400.mat','-100.mat'}, skip, [400 100], [1 1]);
%tstf2 = calcbeatchroma(testFileList,prefix,suffix,scratchDir,'-100.mat', skip, 100);
% add key estimates

%K4 = load('KeyModel400');
%K1 = load('KeyModel100');
%keys = rewrite_key_ests2(tstf,K4.KeyModel,K1.KeyModel,keycheat);

K = load('KeyModel');
keys = rewrite_key_ests_cov(tstf,K.KeyModel,keycheat);

FD = load('ChordPrior');
ChordPrior = FD.ChordPrior;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  test  %%%%%%%%%%%%

mymkdir(resultsDir);

outf = cell(1,length(tstf));

for ti = 1:length(tstf)

  % load test data
  [F,B,TT,K] = loadftrs_mirex(tstf(ti), keynorm); % pass a 1 element cell array

  % output file name
  [pathstr,name,ext] = fileparts(tfl{ti});
  opf = fullfile(resultsDir,[name,ext,'.txt']);

  nf = size(F,2);
  luul2 = 25;  % num models

  pp=[];
  % evaluate each frame under all models
  for j = 1:luul2; 
    pp(j,:) = gaussian_prob(F,M(j).mean,M(j).sigma); 
  end

  %gt = L;

  %ppn = pp./repmat(sum(pp),luul2,1);
  ppnnn = pp(1:end-1,:)./repmat(sum(pp(1:end-1,:)),luul2-1,1);

  %[vv,xx] = max(ppnnn);

  %disp(['raw classification, no N=',num2str(mean(xx==gt))]);

  %imgsc(ppnnn)
  % find viterbi path
  pri = ones(1,luul2-1)/luul2-1;
  %pri = ChordPrior(1:24);
  pri = pri/sum(pri);
%  pth = viterbi_path(pri,txnn,ppnnn);
  % MaxGamma labels instead of a true Viterbi path (a little better)
  [alph,beta,gamm] = fwdback(pri,txnn,ppnnn);
  [vv,pth] = max(gamm);

  %hold on; plot(pth,'-r'); hold off
  
  %noN = (L ~= 25);

  %S = mean(pth(noN)==gt(noN));
  %C = confus(pth(noN),gt(noN));

  %disp(['HMM classification, no N=',num2str(S)]);

  %T = txnn;

  % undo key normalization (shift output labels back into true key)
  nchrom = 12;
  maj = (pth <= nchrom);
  min = (pth > nchrom) & (pth <= 2*nchrom);
  pth(maj) = 1+mod(pth(maj)-1 + K, nchrom);
  pth(min) = 1+nchrom+mod(pth(min)-1-nchrom + K, nchrom);
  
  % write output file
  % (write label states as origin-zero)
  rslt = [B',(pth-1)'];
  %%save(opf,'-ascii',rslt);
  %f = fopen(opf,'w');
  %fprintf(f,'%f %d\n',rslt');
  %fclose(f);
  write_chordlabs(B,(pth-1),opf);

  disp([datestr(rem(now,1),'HH:MM:SS'), ' song ',num2str(ti),' wrote ', ...
        opf]);

  outf{ti} = opf;
  
end

if nargout > 0
  T = outf;
end
if nargout > 1
  K = keys;
end
