function [F,B,T,K] = loadftrs_mirex(A,Trans)
% [F,B,T,K] = loadftrslabs(A,Trans)
%     Read in features for data items defined by a file or cell list.
%     A is the name of a list file defining a set of tracks
%     (or a cell array containing all the file ID strings).
%     Trans = 1 means transpose by recorded key (default 0).
%     F returns as one 12xN matrix of chroma features
%     B returns the beat times within each block.
%     T is a vector of the start frames of each individual track.
%     K is a vector of transpositions applied to each block
% 2008-08-11 Dan Ellis dpwe@ee.columbia.edu

if nargin < 2
  Trans = 0;
end

% optional common filename prefix
dataroot = '';

% Compressive exponent
pwr = 0.25;

if iscell(A)
  trks = A;
else
  trks = listfileread(A);
end

ntrks = length(trks);
F = []; 
B = [];
T = [];
K = [];

lowchroma = 1;
tonctrd = 0;

for i=1:ntrks; 

  fn = fullfile(dataroot, trks{i});
  FF=load(fn);
  if Trans == 0
    key = 0;
  else
    key = FF.key;
  end
  f = chromnorm(chrompwr(chromrot(FF.F, key), pwr));
  if tonctrd
    f = chrom2tonctrd(f);
  end
  if lowchroma
    % convert "-400" into "-100"
    fn2 = fn;
    %fn2(max(find(fn2=='4'))) = '1';
    fn2(end-6) = '1';
    FF2=load(fn2);
    f2 = chromnorm(chrompwr(chromrot(FF2.F, key), pwr));
    if tonctrd
      f2 = chrom2tonctrd(f2);
    end
    f = [f;f2];
  end
  
  F=[F,f];
  B=[B,FF.bts];
  T(i) = size(f,2);
  K(i) = key;
end

